--------------------------------------------------------
-- VHDL code del modulo principale CPU.
-- By Sebastiano Otupacca - PAP1 2008/2009, 08-05-2009
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------

entity CPU is

  port(CK    : in std_logic;   -- Clock
       CLR_N : in std_logic);  -- Clear (active-low)

end CPU;

--------------------------------------------------------

architecture struct of CPU is

  component MEMORY
    port(CK                : in std_logic;                          -- Clock
         CLR_N             : in std_logic;                          -- Clear (active-low)
         rd_N              : in std_logic;                          -- Read enable (active-low)
         wrt_N             : in std_logic;                          -- Write enable (active-low)
         incpc_N           : in std_logic;                          -- Increment PC (active-low)
         ldpc_N            : in std_logic;                          -- Load PC (active-low)
         rdpc_N            : in std_logic;                          -- Read PC (active-low)
         incmar_N          : in std_logic;                          -- Increment MAR - Memory Address Register (active-low)
         ldmar_N           : in std_logic;                          -- Load MAR - Memory Address Register (active-low)
         ldmdr_N           : in std_logic;                          -- Load Memory Data (active-low)
         ldhmdr_N          : in std_logic;                          -- Load Memory Data - High byte (active-low)
         rdmdr_N           : in std_logic;                          -- Read Memory Data
         spc               : in std_logic;                          -- Select PC
         TTY_statusAddrMap : in std_logic_vector(15 downto 0);      -- Indirizzo di mappatura del TTY_Status nella memoria
         TTY_dataAddrMap   : in std_logic_vector(15 downto 0);      -- Indirizzo di mappatura del TTY_Data nella memoria
         RTS_in            : in std_logic;                          -- Request To Send dal TTY
         DTR_in            : in std_logic;                          -- Data Terminal Ready dal TTY
         TTY_data_recv     : in std_logic_vector(7 downto 0);       -- Dati ricevuti dal TTY
         CTS_out           : out std_logic;                         -- Clear To Send verso il TTY
         DSR_out           : out std_logic;                         -- Data Set Ready verso il TTY
         TTY_data_send     : out std_logic_vector(7 downto 0);      -- Dati inviati tramite il TTY
         DATA              : inout std_logic_vector(15 downto 0));  -- Bus dati della CPU
  end component;

  component IUNIT
    port(CK       : in std_logic;                          -- Clock
         CLR_N    : in std_logic;                          -- Clear (active-low)
         LT_N     : in std_logic;
         GT_N     : in std_logic;
         Z_N      : in std_logic;
         COUT     : in std_logic;
         rd_N     : out std_logic;                         -- Read enable (active-low)
         wrt_N    : out std_logic;                         -- Write enable (active-low)
         incpc_N  : out std_logic;                         -- Increment PC (active-low)
         ldpc_N   : out std_logic;                         -- Load PC (active-low)
         rdpc_N   : out std_logic;                         -- Read PC (active-low)
         incmar_N : out std_logic;                         -- Increment MAR - Memory Address Register (active-low)
         ldmar_N  : out std_logic;                         -- Load MAR - Memory Address Register (active-low)
         ldmdr_N  : out std_logic;                         -- Load Memory Data (active-low)
         ldhmdr_N : out std_logic;                         -- Load Memory Data - High byte (active-low)
         rdmdr_N  : out std_logic;                         -- Read Memory Data
         spc      : out std_logic;                         -- Select PC
         SA       : out std_logic_vector(3 downto 0);
         SB       : out std_logic_vector(3 downto 0);
         WA_N     : out std_logic;
         AOP      : out std_logic_vector(1 downto 0);
         BOP      : out std_logic_vector(1 downto 0);
         FUNC     : out std_logic_vector(4 downto 0);
         IDATA    : out std_logic_vector(7 downto 0);
         CLQ_N    : out std_logic;
         LDQ_N    : out std_logic;
         CIN      : out std_logic;
         LDDATA_N : out std_logic;
         LDQH_N   : out std_logic;
         LDQL_N   : out std_logic;
         DOUT_N   : out std_logic;
         DATA     : inout std_logic_vector(15 downto 0));  -- Bus dati della CPU
  end component;

  component EUNIT
    port(CK       : in std_logic;                          -- Clock
         CLR_N    : in std_logic;                          -- Clear (active-low)
         SA       : in std_logic_vector(3 downto 0);
         SB       : in std_logic_vector(3 downto 0);
         WA_N     : in std_logic;
         AOP      : in std_logic_vector(1 downto 0);
         BOP      : in std_logic_vector(1 downto 0);
         FUNC     : in std_logic_vector(4 downto 0);
         IDATA    : in std_logic_vector(7 downto 0);
         CLQ_N    : in std_logic;
         LDQ_N    : in std_logic;
         CIN      : in std_logic;
         LDDATA_N : in std_logic;
         LDQH_N   : in std_logic;
         LDQL_N   : in std_logic;
         DOUT_N   : in std_logic;
         LT_N     : out std_logic;
         GT_N     : out std_logic;
         Z_N      : out std_logic;
         COUT     : out std_logic;
         DATA     : inout std_logic_vector(15 downto 0));  -- Bus dati della CPU
  end component;

  -- Segnali interni
  signal rd_N     : std_logic;
  signal wrt_N    : std_logic;
  signal incpc_N  : std_logic;
  signal ldpc_N   : std_logic;
  signal rdpc_N   : std_logic;
  signal incmar_N : std_logic;
  signal ldmar_N  : std_logic;
  signal ldmdr_N  : std_logic;
  signal ldhmdr_N : std_logic;
  signal rdmdr_N  : std_logic;
  signal WA_N     : std_logic;
  signal CLQ_N    : std_logic;
  signal LDQ_N    : std_logic;
  signal CIN      : std_logic;
  signal LDDATA_N : std_logic;
  signal LDQH_N   : std_logic;
  signal LDQL_N   : std_logic;
  signal DOUT_N   : std_logic;
  signal LT_N     : std_logic;
  signal GT_N     : std_logic;
  signal Z_N      : std_logic;
  signal COUT     : std_logic;
  signal spc      : std_logic;
  signal RTS      : std_logic;
  signal DTR      : std_logic;
  signal CTS      : std_logic;
  signal DSR      : std_logic;

  signal SA                : std_logic_vector(3 downto 0);
  signal SB                : std_logic_vector(3 downto 0);
  signal AOP               : std_logic_vector(1 downto 0);
  signal BOP               : std_logic_vector(1 downto 0);
  signal FUNC              : std_logic_vector(4 downto 0);
  signal IDATA             : std_logic_vector(7 downto 0);
  signal DATA              : std_logic_vector(15 downto 0);

  -- TTY
  signal TTY_statusAddrMap : std_logic_vector(15 downto 0);
  signal TTY_dataAddrMap   : std_logic_vector(15 downto 0);
  signal TTY_data_recv     : std_logic_vector(7 downto 0);
  signal TTY_data_send     : std_logic_vector(7 downto 0);

begin

  -- Istanzia il modulo MEMORY
  blk_U1: MEMORY port map(CK                => CK,
                          CLR_N             => CLR_N,
                          rd_N              => rd_N,
                          wrt_N             => wrt_N,
                          incpc_N           => incpc_N,
                          ldpc_N            => ldpc_N,
                          rdpc_N            => rdpc_N,
                          incmar_N          => incmar_N,
                          ldmar_N           => ldmar_N,
                          ldmdr_N           => ldmdr_N,
                          ldhmdr_N          => ldhmdr_N,
                          rdmdr_N           => rdmdr_N,
                          spc               => spc,
                          TTY_statusAddrMap => TTY_statusAddrMap,
                          TTY_dataAddrMap   => TTY_dataAddrMap,
                          RTS_in            => RTS,
                          DTR_in            => DTR,
                          TTY_data_recv     => TTY_data_recv,
                          CTS_out           => CTS,
                          DSR_out           => DSR,
                          TTY_data_send     => TTY_data_send,
                          DATA              => DATA);

  -- Istanzia il modulo IUNIT
  blk_U2: IUNIT port map(CK       => CK,
                         CLR_N    => CLR_N,
                         LT_N     => LT_N,
                         GT_N     => GT_N,
                         Z_N      => Z_N,
                         COUT     => COUT,
                         rd_N     => rd_N,
                         wrt_N    => wrt_N,
                         incpc_N  => incpc_N,
                         ldpc_N   => ldpc_N,
                         rdpc_N   => rdpc_N,
                         incmar_N => incmar_N,
                         ldmar_N  => ldmar_N,
                         ldmdr_N  => ldmdr_N,
                         ldhmdr_N => ldhmdr_N,
                         rdmdr_N  => rdmdr_N,
                         spc      => spc,
                         SA       => SA,
                         SB       => SB,
                         WA_N     => WA_N,
                         AOP      => AOP,
                         BOP      => BOP,
                         FUNC     => FUNC,
                         IDATA    => IDATA,
                         CLQ_N    => CLQ_N,
                         LDQ_N    => LDQ_N,
                         CIN      => CIN,
                         LDDATA_N => LDDATA_N,
                         LDQH_N   => LDQH_N,
                         LDQL_N   => LDQL_N,
                         DOUT_N   => DOUT_N,
                         DATA     => DATA);

  -- Istanzia il modulo EUNIT
  blk_U3: IUNIT port map(CK       => CK,
                         CLR_N    => CLR_N,
                         SA       => SA,
                         SB       => SB,
                         WA_N     => WA_N,
                         AOP      => AOP,
                         BOP      => BOP,
                         FUNC     => FUNC,
                         IDATA    => IDATA,
                         CLQ_N    => CLQ_N,
                         LDQ_N    => LDQ_N,
                         CIN      => CIN,
                         LDDATA_N => LDDATA_N,
                         LDQH_N   => LDQH_N,
                         LDQL_N   => LDQL_N,
                         DOUT_N   => DOUT_N,
                         LT_N     => LT_N,
                         GT_N     => GT_N,
                         Z_N      => Z_N,
                         COUT     => COUT,
                         DATA     => DATA);

end struct;

--------------------------------------------------------
