--------------------------------------------------------
-- VHDL code per modulo RAM ad nData-bit di dati e
-- nAddress-bit di indirizzi.
-- By Sebastiano Otupacca - PAP1 2008/2009, 30-04-2009
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

--------------------------------------------------------

entity RAM is

  generic(nData: natural := 8; nAddress: natural := 16);
  port(A    : in std_logic_vector(nAddress-1 downto 0);   -- Bus Indirizzi
       CS_N : in std_logic;                               -- Chip Select (active-low)
       WE_N : in std_logic;                               -- Write Enable (active-low)
       OE_N : in std_logic;                               -- Output Enable (active-low)
       D    : inout std_logic_vector(nData-1 downto 0));  -- Bus Dati

end RAM;

--------------------------------------------------------

architecture rtl of RAM is

  -- Istanzia la memoria con un array di 2**nAddress elementi a nData bit
  type arrVector is array (2**nAddress-1 downto 0) of std_logic_vector(nData-1 downto 0);
  signal RAM_memory : arrVector;

begin

  -- Lettura dell'elemento di RAM_vector puntato dall'indirizzo 'A' index
  D <= RAM_memory(conv_integer(A)) when CS_N = '0' and OE_N = '0' and WE_N = '1' else
       (others => 'Z');

  -- Scrittura dell'elemento di RAM_vector puntato dall'indirizzo 'A' index
  RAM_memory(conv_integer(A)) <= D when CS_N = '0' and OE_N = '1' and WE_N = '0';

end rtl;

--------------------------------------------------------
