--------------------------------------------------------
-- VHDL code per il TestBench del modulo RAM.
-- By Sebastiano Otupacca - PAP1 2008/2009, 30-04-2009
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------

entity RAM_tb is

end RAM_tb;

--------------------------------------------------------

architecture test of RAM_tb is

  component RAM
    generic(nData: natural := 8; nAddress: natural := 16);
    port(A    : in std_logic_vector(nAddress-1 downto 0);
         CS_N : in std_logic;
         WE_N : in std_logic;
         OE_N : in std_logic;
         D    : inout std_logic_vector(nData-1 downto 0));
  end component;

  -- component generics
  constant nData : natural := 8;
  constant nAddress : natural := 16;

  -- component ports
  signal A    : std_logic_vector(nAddress-1 downto 0);
  signal CS_N : std_logic;
  signal WE_N : std_logic;
  signal OE_N : std_logic;
  signal D    : std_logic_vector(nData-1 downto 0);

  constant step : time := 10 ns;

begin

  -- component instantiation
  RAMcomp: RAM
    generic map(nData => nData, nAddress => nAddress)
    port map(A    => A,
             CS_N => CS_N,
             WE_N => WE_N,
             OE_N => OE_N,
             D    => D);

  test_proc: process

  begin
    -- Scrivi il valore 103 (0x67) nell'indirizzo 13 (0x0d)
    A <= "0000000000001101";  -- 13 (0x0d)
    D <= "01100111";          -- 103 (0x67)
    CS_N <= '0';
    WE_N <= '1';
    OE_N <= '1';
    wait for step;
    WE_N <= '0';
    wait for step;
    WE_N <= '1';
    wait for step;

    -- Scrivi il valore 167 (0xa7) nell'indirizzo 5 (0x05)
    A <= "0000000000000101";  -- 5
    D <= "10100111";          -- 167
    wait for step;
    WE_N <= '0';
    wait for step;
    WE_N <= '1';
    wait for step;

    -- Scrivi il valore 41 (0x29) nell'indirizzo 13 (0x0b)
    A <= "0000000000001011";  -- 11
    D <= "00101001";          -- 41
    wait for step;
    WE_N <= '0';
    wait for step;
    WE_N <= '1';
    wait for step;

    -- Scrivi il valore 195 (0xc3) nell'indirizzo 9 (0x09)
    A <= "0000000000001001";  -- 9
    D <= "11000011";          -- 195
    wait for step;
    WE_N <= '0';
    wait for step;
    WE_N <= '1';
    wait for step;

    -- Scrivi il valore 38 (0x26) nell'indirizzo 26 (0x19)
    A <= "0000000000011001";  -- 25
    D <= "00100110";          -- 38
    wait for step;
    WE_N <= '0';
    wait for step;
    WE_N <= '1';
    D <= "ZZZZZZZZ";  -- Bus di dati in alta impedenza
    wait for step;

    -- Leggi il dato dell'indirizzo 13 (0x0d), valore non ritornato
    -- a causa della combinazione proibita (OE e WE entrambi attivi)
    A <= "0000000000001101";  -- 13
    wait for step;
    OE_N <= '0';  -- OE e WE attivi = Combinazione proibita
    WE_N <= '0';  -- la memoria mantiene il bus dei dati in alta impedenza
    wait for step;
    OE_N <= '1';
    WE_N <= '1';
    wait for step;

    -- Leggi il dato dell'indirizzo 13 (0x0d)
    OE_N <= '0';
    wait for step;
    OE_N <= '1';
    wait for step;

    -- Leggi il dato dell'indirizzo 5 (0x05)
    A <= "0000000000000101";  -- 5
    wait for step;
    OE_N <= '0';
    wait for step;
    OE_N <= '1';
    wait for step;

    -- Leggi il dato dell'indirizzo 11 (0x0b)
    A <= "0000000000001011";  -- 11
    wait for step;
    OE_N <= '0';
    wait for step;
    OE_N <= '1';
    wait for step;

    -- Leggi il dato dell'indirizzo 9 (0x09)
    A <= "0000000000001001";  -- 9
    wait for step;
    OE_N <= '0';
    wait for step;
    OE_N <= '1';
    wait for step;

    -- Leggi il dato dell'indirizzo 25 (0x19)
    A <= "0000000000011001";  -- 25
    wait for step;
    OE_N <= '0';
    wait for step;
    OE_N <= '1';
    wait for step;

    wait;
  end process test_proc;

end test;

--------------------------------------------------------
