--------------------------------------------------------
-- VHDL code per Registro con clock.
-- By Sebastiano Otupacca - PAP1 2008/2009, 30-04-2009
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------

entity FFD is

  port(clk   : in std_logic;    -- Clock
       CLR_N : in std_logic;    -- Reset (active-low, asincrono)
       EN_N  : in std_logic;    -- Enable (active-low)
       D     : in std_logic;    -- Data input
       Q     : out std_logic);  -- Data output

end FFD;

--------------------------------------------------------

architecture rtl of FFD is

begin

  Q <= '0' when CLR_N = '0' else
       D when CLR_N = '1' and EN_N = '0' and clk'event and clk = '1' and clk'last_value = '0';

end rtl;

--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------

entity Reg is

  generic(n: natural := 8);
  port(clk   : in std_logic;                         -- Clock
       CLR_N : in std_logic;                         -- Reset (active-low, asincrono)
       EN_N  : in std_logic;                         -- Enable (active-low)
       D     : in std_logic_vector(n-1 downto 0);    -- Data input
       Q     : out std_logic_vector(n-1 downto 0));  -- Data output

end Reg;

--------------------------------------------------------

architecture struct of Reg is

  component FFD
    port(clk   : in std_logic;    -- Clock
         CLR_N : in std_logic;    -- Reset (active-low, asincrono)
         EN_N  : in std_logic;    -- Enable (active-low)
         D     : in std_logic;    -- Data input
         Q     : out std_logic);  -- Data output
  end component;

begin

  -- Istanzia n volte la cella 'FFD' ad 1 bit
  genReg: for i in 0 to n-1 generate
            cell: FFD port map(clk   => clk,
                               CLR_N => CLR_N,
                               EN_N  => EN_N,
                               D     => D(i),
                               Q     => Q(i));
  end generate;

end struct;

--------------------------------------------------------
