--------------------------------------------------------
-- VHDL code per il TestBench del modulo REG4.
-- By Sebastiano Otupacca - PAP1 2008/2009, 09-05-2009
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------

entity REG4_tb is

end REG4_tb;

--------------------------------------------------------

architecture test of REG4_tb is

  component REG4
    port(CK    : in std_logic;                        -- Clock
         CLR_N : in std_logic;                        -- Clear (active-low)
         WA_N  : in std_logic;                        -- Write A (active-low)
         ENA_N : in std_logic;                        -- Enable A (active-low)
         ENB_N : in std_logic;                        -- Enable B (active-low)
         SA    : in std_logic_vector(1 downto 0);     -- Select A
         SB    : in std_logic_vector(1 downto 0);     -- Select B
         DIN   : in std_logic_vector(15 downto 0);    -- Data input
         A     : out std_logic_vector(15 downto 0);   -- A output
         B     : out std_logic_vector(15 downto 0));  -- B output
  end component;

  -- component ports
  signal clk   : std_logic;
  signal CLR_N : std_logic;
  signal WA_N  : std_logic;
  signal ENA_N : std_logic;
  signal ENB_N : std_logic;
  signal SA    : std_logic_vector(1 downto 0);
  signal SB    : std_logic_vector(1 downto 0);
  signal DIN   : std_logic_vector(15 downto 0);
  signal A     : std_logic_vector(15 downto 0);
  signal B     : std_logic_vector(15 downto 0);

  -- Disabilitazione clock
  signal clkStop : std_logic;

  constant step : time := 10 ns;

begin

  -- component instantiation
  REG4comp: REG4
    port map(CK    => clk,
             CLR_N => CLR_N,
             WA_N  => WA_N,
             ENA_N => ENA_N,
             ENB_N => ENB_N,
             SA    => SA,
             SB    => SB,
             DIN   => DIN,
             A     => A,
             B     => B);

  clk_proc: process
  begin
    if clkStop = '1' then
      clk <= '0';
      wait;
    else
      clk <= '0';
      wait for 2.7 ns;
      clk <= '1';
      wait for 2.1 ns;
    end if;
  end process;

  test_proc: process

  begin
    clkStop <= '0';
    CLR_N <= '0';
    WA_N <= '0';
    ENA_N <= '1';
    ENB_N <= '1';
    SA <= "01";
    SB <= "10";
    DIN <= "0000000010011101";
    wait for step;

    CLR_N <= '1';
    ENA_N <= '0';
    ENB_N <= '0';
    ENA_N <= '0';
    wait for step;

    ENA_N <= '1';
    wait for step;

    SA <= "00";
    ENA_N <= '0';
    DIN <= "0000000001001011";
    wait for step;

    ENA_N <= '1';
    wait for step;

    SA <= "11";
    ENA_N <= '0';
    DIN <= "0000000110010110";
    wait for step;

    ENA_N <= '1';
    wait for step;

    SA <= "10";
    ENA_N <= '0';
    DIN <= "0000110010010101";
    wait for step;

    WA_N <= '1';
    SA <= "00";
    SB <= "01";
    wait for step;

    SA <= "01";
    SB <= "10";
    wait for step;

    SA <= "10";
    SB <= "11";
    wait for step;

    SA <= "11";
    SB <= "00";
    wait for step;
    

    clkStop <= '1';
    wait;
  end process test_proc;

end test;

--------------------------------------------------------
