--------------------------------------------------------
-- VHDL code per il TestBench del registro con clock.
-- By Sebastiano Otupacca - PAP1 2008/2009, 30-04-2009
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

--------------------------------------------------------

entity Reg_tb is

end Reg_tb;

--------------------------------------------------------

architecture test of Reg_tb is

  component Reg
    generic(n: natural := 8);
    port(clk   : in std_logic;
         CLR_N : in std_logic;
         EN_N  : in std_logic;
         D     : in std_logic_vector(n-1 downto 0);
         Q     : out std_logic_vector(n-1 downto 0));
  end component;

  -- component generics
  constant n : natural := 8;

  -- component ports
  signal clk   : std_logic;
  signal CLR_N : std_logic;
  signal EN_N  : std_logic;
  signal D     : std_logic_vector(n-1 downto 0);
  signal Q     : std_logic_vector(n-1 downto 0);

  constant step : time := 10 ns;

begin

  -- component instantiation
  Regcomp: Reg
    generic map(n => n)
    port map(clk   => clk,
             CLR_N => CLR_N,
             EN_N  => EN_N,
             D     => D,
             Q     => Q);

  test_proc: process

  begin
    -- Reset
    clk <= '0';
    CLR_N <= '0';
    EN_N <= '0';
    wait for step;
    CLR_N <= '1';
    wait for step;

    -- Itera i 256 valori possibili con 8 bit di dati
    for i in 0 to 2**n-1 loop
      -- Disabilita il registro per i valori 10, 11, 12 e 13
      if i >= 10 and i <= 13 then CLR_N <= '0';
      else CLR_N <= '1';
      end if;

      -- Test di transizione del clock da 'X' a '1'
      if i = 19 then
        clk <= 'X';
      end if;

      D <= conv_std_logic_vector(i, n);
      EN_N <= D(0);  -- EN_N => toggle tra valori di 'i' pari e dispari
      wait for step;

      clk <= '1';
      wait for step;

      clk <= '0';
      wait for step;
    end loop;

    wait;
  end process test_proc;

end test;

--------------------------------------------------------
