--------------------------------------------------------
-- Title : VHDL testbench per mem
-- Project : Menagerie CPU
--------------------------------------------------------
-- File : mem_tb.vhdl
-- Author : Giovanni Franza
-- Company : Studio Informatico Cabrini e Franza
-- Created : 2009-05-07
-- Last Update: 2009-05-07
-- Platform :
-- Standard : VHDL'87
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.mempack.all;

-------------------------------------------------------------------------------

entity mem_tb is
end mem_tb;

-------------------------------------------------------------------------------

architecture test of mem_tb is

  component m11

    port(
      A : in  std_logic_vector(7 downto 0);  -- Address
      D : out std_logic_vector(31 downto 0)  -- Data output
      );

  end component;

  component m21

    port(
      A : in  std_logic_vector(7 downto 0);  -- Address
      D : out std_logic_vector(31 downto 0)  -- Data output
      );

  end component;

  component map11

    port(
      A : in  std_logic_vector(7 downto 0);  -- Address
      D : out std_logic_vector(7 downto 0)   -- Data output
      );

  end component;

  -- component ports

  constant n : natural := 8;

  signal A_i   : std_logic_vector(7 downto 0);  -- Address
  signal D32_i : std_logic_vector(7 downto 0);  -- Data output
  signal D8_i  : std_logic_vector(7 downto 0);  -- Data output

  signal abilitato  : std_logic := '0';  -- Attiva/disattiva il clock
  signal orologio_i : std_logic := '0';  -- clock

  constant step : time := 7 ns;

begin  -- test

  m1_1 : m11

    port map (
      A => A_i,
      D => D32_i
      );

  m2_1 : m21

    port map (
      A => A_i,
      D => D32_i
      );

  map1_1 : map11

    port map (
      A => A_i,
      D => D8_i
      );

  -- il clock per la macchina Ã¨ fatto in modo che possa
  -- essere 'spento' quando non necessario altrimenti la
  -- visualizzazione delle tracce viene male

  pendola : process
    constant duty_on  : time := 1 ns;
    constant duty_off : time := 1 ns;
  begin
    if abilitato = '0' then
      wait until abilitato = '1';
    end if;
    orologio_i <= '0';
    wait for duty_on;
    orologio_i <= '1';
    wait for duty_off;

  end process;  -- pendola

  stim : process

  begin

    abilitato <= '1';

    ---

    abilitato <= '0';
    report "Lavoro finito";
    wait;

  end process stim;

end test;

-------------------------------------------------------------------------------
configuration mem_tb_test_cfg of mem_tb is

  for test
    for m1_1 : m11
      use entity work.m1(RTL);
    end for;
    for m2_1 : m21
      use entity work.m2(RTL);
    end for;
    for map1_1 : map11
      use entity work.map1(RTL);
    end for;
  end for;

end mem_tb_test_cfg;
